@@ -1,8 +1,9 @@
-from harpoon.errors import HarpoonError, BadOption, BadDockerConnection
+from harpoon.errors import BadOption, BadDockerConnection
 from harpoon.overview import Harpoon
 
 from rainbow_logging_handler import RainbowLoggingHandler
 from docker.client import Client as DockerClient
+from delfick_error import DelfickError
 import requests
 import argparse
 import logging
@@ -90,8 +91,13 @@ class CliParser(object):
         opts = {}
         if os.path.exists("./harpoon.yml"):
             opts["default"] = "./harpoon.yml"
+            opts["required"] = False
         else:
             opts["required"] = True
+
+        if "HARPOON_CONFIG" in os.environ:
+            opts["default"] = os.environ["HARPOON_CONFIG"]
+            del opts["required"]
         parser.add_argument("--harpoon-config"
             , help = "The config file specifying what harpoon should care about"
             , type = argparse.FileType("r")
@@ -179,7 +185,7 @@ def main(argv=None):
         kwargs = vars(args)
         kwargs["extra"] = extra
         Harpoon(configuration_file=args.harpoon_config.name, docker_context=docker_context(), interactive=args.interactive, silent_build=args.silent_build, logging_handler=handler).start(**kwargs)
-    except HarpoonError as error:
+    except DelfickError as error:
         print ""
         print "!" * 80
         print "Something went wrong! -- {0}".format(error.__class__.__name__)
